@@ -9,11 +9,12 @@
  * Copyright 2001, Red Hat, Inc., Ximian, Inc.,
  *                 Sun Microsystems, Inc.
  */
+#include <config.h>
+
 #include <stdio.h>
 #include <errno.h>
 #include <unistd.h>
 #include <signal.h>
-#include "linc-debug.h"
 #include "linc-private.h"
 
 /* whether we do locking or not */
