@@ -186,7 +186,8 @@ ORBit_small_get_iinterface (CORBA_Object       opt_object,
 
 static void
 add_if_unique (GPtrArray  *strings,
-	       const char *new_str)
+	       const char *new_str,
+	       gboolean base_path)
 {
 	int i, len;
 
@@ -200,7 +201,9 @@ add_if_unique (GPtrArray  *strings,
 
 	g_ptr_array_add (
 		strings,
-		g_strconcat (new_str, "/lib/orbit-2.0", NULL));
+		base_path ? 
+		  g_strconcat (new_str, "/lib/orbit-2.0", NULL) : 
+    		  g_strdup(new_str));
 }
 
 /* FIXME: this should be called only once at
@@ -221,16 +224,16 @@ ORBit_get_typelib_paths (void)
 
 		strv = g_strsplit (path, ":", -1);
 		for (i = 0; strv && strv [i]; i++)
-			add_if_unique (paths, strv [i]);
+			add_if_unique (paths, strv [i], FALSE);
 		g_strfreev (strv);
 	}
 
-	if ((path = g_getenv ("GNOME_PATH"))) {
+	if ((path = g_getenv ("GNOME2_PATH"))) {
 		char **strv;
 
 		strv = g_strsplit (path, ":", -1);
 		for (i = 0; strv && strv [i]; i++)
-			add_if_unique (paths, strv [i]);
+			add_if_unique (paths, strv [i], TRUE);
 		g_strfreev (strv);
 	}
 
@@ -246,10 +249,8 @@ load_module (const char *fname, const char *libname)
 	ORBit_IModule *module;
 
 	if (!(handle = g_module_open (fname, G_MODULE_BIND_LAZY))) {
-#ifdef TYPE_DEBUG
-		g_warning ("Failed to load '%s': '%s'", fname,
-			   g_module_error ());
-#endif
+		dprintf (TYPES, "Failed to load '%s': '%s'\n", fname,
+			 g_module_error ());
 		return FALSE;
 	}
 
@@ -265,10 +266,8 @@ load_module (const char *fname, const char *libname)
 		ORBit_IInterface **p;
 		gulong length, i;
 
-#ifdef TYPE_DEBUG
-		g_warning ("Loaded interfaces of serial %d from '%s'",
+		dprintf (TYPES, "Loaded interfaces of serial %d from '%s'\n",
 			   module->version, fname);
-#endif
 
 		for (p = module->interfaces, length = 0; p && *p; p++)
 			length++;
@@ -290,9 +289,7 @@ load_module (const char *fname, const char *libname)
 			dest = &iinterfaces->_buffer [i];
 			add_iinterface (dest);
 
-#ifdef TYPE_DEBUG
-			g_warning ("Type '%s'", dest->tc->repo_id);
-#endif
+			dprintf (TYPES, "Type '%s'\n", dest->tc->repo_id);
 		}
 
 		types = ORBit_copy_value (
@@ -311,6 +308,17 @@ load_module (const char *fname, const char *libname)
 }
 
 
+/**
+ * ORBit_small_load_typelib:
+ * @libname: the name of the type library to load
+ * 
+ * This method has security issues if you do not use
+ * an absolute path in @libname. The environment variables
+ * ORBIT_TYPELIB_PATH and GNOME2_PATH are used to scan for
+ * type libraries.
+ * 
+ * Return value: FALSE if load failed.
+ **/
 gboolean
 ORBit_small_load_typelib (const char *libname)
 {
