@@ -37,11 +37,13 @@
 #ifdef TIMING_RUN
 #  define d_print(a)
 #else
-#  define d_print(a) g_print(a)
+#  define d_print(a) if (!thread_tests) g_print(a)
 #endif
 
 extern CORBA_ORB global_orb;
 gboolean         in_proc;
+gboolean         thread_safe = FALSE;
+gboolean         thread_tests = FALSE;
 
 /* Ugly- but hey */
 #define _IN_CLIENT_
@@ -69,6 +71,8 @@ testSequenceHelpers (void)
 	CORBA_long           l    = 0;
 	test_BoundedLongSeq *lseq = NULL;
 
+	d_print ("Testing ORBit_sequence helpers...\n");
+
 	lseq = ORBit_sequence_alloc (TC_test_BoundedLongSeq, 2);
 	g_assert (lseq != NULL);
 	g_assert (lseq->_length ==  2);
@@ -86,6 +90,10 @@ testSequenceHelpers (void)
 	g_assert (ORBit_sequence_index (lseq, 2) == 1);
 	g_assert (ORBit_sequence_index (lseq, 3) == 2);
 
+        ORBit_sequence_remove(lseq, 2);
+	g_assert (lseq->_length == 3);
+	g_assert (ORBit_sequence_index (lseq, 2) == 2);
+
 	ORBit_sequence_set_size (lseq, 100);
 	ORBit_sequence_set_size (lseq, 0);
 
@@ -360,6 +368,19 @@ testException (test_BasicServer   objref,
 	CORBA_free (cpyev);
 }
 
+static void
+testBoolAlign (test_BasicServer   objref,
+	       CORBA_Environment *ev)
+{
+	char *inoutArg;
+
+	d_print ("Testing bool arg. alignment ...\n");
+
+	inoutArg = CORBA_string_dup("foo");
+	test_BasicServer_testBoolString (objref, TRUE, "retout", &inoutArg, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
 static CORBA_TypeCode
 find_tc (CORBA_sequence_CORBA_TypeCode *tcs, 
 	 const char                    *repo_id)
@@ -584,6 +605,49 @@ testCompoundStruct (test_TestFactory   factory,
   g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
+static void
+testAlignHoleStruct (test_TestFactory   factory, 
+		    CORBA_Environment *ev)
+{
+  test_StructServer objref;
+  test_AlignHoleStruct inArg, inoutArg, outArg, retn;
+  d_print ("Testing structs with aligning holes...\n");
+  objref = test_TestFactory_getStructServer (factory, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+  inArg.a.a = constants_DOUBLE_IN;
+  inArg.a.b = constants_OCTET_IN;
+  inArg.b = constants_CHAR_IN;
+
+  inoutArg.a.a = constants_DOUBLE_INOUT_IN;
+  inoutArg.a.b = constants_OCTET_INOUT_IN;
+  inoutArg.b = constants_CHAR_INOUT_IN;
+  
+  memset(&outArg, 0, sizeof(outArg));
+
+  retn = test_StructServer_opAlignHole (objref, &inArg, &inoutArg, &outArg, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+  
+  g_assert (inArg.a.a == constants_DOUBLE_IN);
+  g_assert (inArg.a.b == constants_OCTET_IN);
+  g_assert (inArg.b == constants_CHAR_IN);
+
+  g_assert (inoutArg.a.a == constants_DOUBLE_INOUT_OUT);
+  g_assert (inoutArg.a.b == constants_OCTET_INOUT_OUT);
+  g_assert (inoutArg.b == constants_CHAR_INOUT_OUT);
+
+  g_assert (outArg.a.a == constants_DOUBLE_OUT);
+  g_assert (outArg.a.b == constants_OCTET_OUT);
+  g_assert (outArg.b == constants_CHAR_OUT);
+
+  g_assert (retn.a.a == constants_DOUBLE_RETN);	
+  g_assert (retn.a.b == constants_OCTET_RETN);	
+  g_assert (retn.b == constants_CHAR_RETN);	
+  
+  CORBA_Object_release (objref, ev);
+  g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
 static void
 testObjectStruct (test_TestFactory   factory, 
 		  CORBA_Environment *ev)
@@ -1537,6 +1601,72 @@ testMisc (test_TestFactory   factory,
 	}
 }
 
+static void
+testIOR (test_TestFactory   factory, 
+	 CORBA_Environment *ev)
+{
+	int i, count;
+	CORBA_Object objref;
+
+	d_print ("Testing IOR marshalling ...\n");
+
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	/* Check to see that the ORB correctly marshals various obj references */
+	count = test_BasicServer_getObjectCount (objref, ev);
+	for (i = 0; i < count; i++)
+	{
+		CORBA_Object test_obj;
+		test_obj = test_BasicServer_getObject (objref, i, ev);
+		if (ev->_major != CORBA_NO_EXCEPTION)
+			g_error ("Error demarshalling object number %d: %s",
+				 i, CORBA_exception_id (ev));
+	}
+
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
+static void
+testLifeCycle (test_TestFactory   factory, 
+	       CORBA_Environment *ev)
+{
+	test_LifeCycleServer objref;
+	ORBit_IMethod       *method;
+
+	d_print ("Testing LifeCycle bits...\n");
+
+	objref = test_TestFactory_createLifeCycleServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+		
+	d_print (" pre gnome 2.4 stubs ...\n");
+
+	if (in_proc) {
+		method = &test_LifeCycleServer__iinterface.methods._buffer[1];
+		g_assert (!strcmp (method->name, "deactivateUnrefOnReturn"));
+	} else {
+		method = &test_LifeCycleServer__iinterface.methods._buffer[0];
+		g_assert (!strcmp (method->name, "deactivateOnReturn"));
+	}
+	
+	ORBit_small_invoke_stub (objref, method, NULL, NULL, NULL, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	if (!in_proc)
+		CORBA_Object_release (objref, ev);
+
+	d_print (" post gnome 2.4 stubs ...\n");
+
+	objref = test_TestFactory_createLifeCycleServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	test_LifeCycleServer_deactivateOnReturn (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	CORBA_Object_release (objref, ev);
+}
+
 static volatile int done = 0;
 
 static void
@@ -1716,6 +1846,7 @@ testPingPong (test_TestFactory   factory,
 	
 	before_remote_hash = CORBA_Object_hash (l_objref, 0, ev);
 
+
 	test_PingPongServer_pingPong (r_objref, l_objref, 64, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	
@@ -1825,6 +1956,42 @@ testPingPong (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
+static void
+testPolicy (test_TestFactory   factory,
+	    gboolean           thread_tests,
+	    CORBA_Environment *ev)
+{
+	ORBitPolicy *policy;
+	test_PingPongServer r_objref, l_objref;
+
+	if (thread_tests || !thread_safe)
+		return;
+
+	d_print ("Testing policy code ...\n");
+	r_objref = test_TestFactory_createPingPongServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	l_objref = TestFactory_createPingPongServer (NULL, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (l_objref, ev); /* only want 1 ref */
+	g_assert (ORBit_small_get_servant (l_objref) != NULL);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+	policy = ORBit_policy_new (ORBIT_TYPE_POLICY_EX,
+				   "allow", global_poa, NULL);
+	ORBit_object_set_policy (r_objref, policy);
+	test_PingPongServer_pingPong (r_objref, l_objref, 64, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	ORBit_object_set_policy (r_objref, NULL);
+	ORBit_policy_unref (policy);
+
+	CORBA_Object_release (l_objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	CORBA_Object_release (r_objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+}
+
 static void
 dummy_cb (LinkConnection *connection, gboolean *invoked)
 {
@@ -1940,6 +2107,8 @@ test_time_noop (test_TestFactory   factory,
 	fprintf (stderr, "In proc (slow) took %g msecs\n",
 		 g_timer_elapsed (timer, NULL) * 1000);
 	ORBit_small_flags = old_flags;
+
+	g_timer_destroy (timer);
 }
 
 static void
@@ -1963,6 +2132,7 @@ test_basic_server (test_TestFactory   factory,
 	testLongDouble (objref, ev);
 	testEnum       (objref, ev);
 	testException  (objref, ev);
+	testBoolAlign  (objref, ev);
 
 	CORBA_Object_release (objref, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
@@ -2035,6 +2205,32 @@ testNonExistent (test_TestFactory factory, CORBA_Environment *ev)
 	CORBA_Object_release (non_existent, NULL);
 }
 
+static void
+testWithException (test_TestFactory factory, CORBA_Environment *ev)
+{
+	int old_flags;
+	CORBA_Object objref;
+
+	CORBA_exception_set (ev, CORBA_SYSTEM_EXCEPTION,
+			     ex_CORBA_OBJECT_NOT_EXIST, NULL);
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	old_flags = ORBit_small_flags;
+	ORBit_small_flags |= ORBIT_SMALL_FORCE_GENERIC_MARSHAL;
+
+	CORBA_exception_set (ev, CORBA_SYSTEM_EXCEPTION,
+			     ex_CORBA_OBJECT_NOT_EXIST, NULL);
+	objref = test_TestFactory_getBasicServer (factory, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	CORBA_Object_release (objref, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	ORBit_small_flags = old_flags;
+}
+
 static void
 run_tests (test_TestFactory   factory, 
 	   gboolean           thread_tests,
@@ -2050,6 +2246,7 @@ run_tests (test_TestFactory   factory,
 		testFixedLengthStruct (factory, ev);
 		testVariableLengthStruct (factory, ev);
 		testCompoundStruct (factory, ev);
+		testAlignHoleStruct (factory, ev);
 		testObjectStruct (factory, ev);
 		testStructAny (factory, ev);
 		testUnboundedSequence (factory, ev);
@@ -2075,9 +2272,14 @@ run_tests (test_TestFactory   factory,
 		if (!thread_tests)
 			testAsync (factory, ev);
 #endif
-		if (!in_proc)
+		if (!in_proc) {
 			testPingPong (factory, thread_tests, ev);
+			testPolicy (factory, thread_tests, ev);
+		}
 		testMisc (factory, ev);
+		testIOR (factory, ev);
+		testWithException (factory, ev);
+		testLifeCycle (factory, ev);
 	}
 	
 #if NUM_RUNS > 1
@@ -2163,8 +2365,6 @@ main (int argc, char *argv [])
 	test_TestFactory   factory;
 	ORBit_IInterfaces *interfaces = NULL;
 	gboolean           gen_imodule = FALSE;
-	gboolean           thread_safe = FALSE;
-	gboolean           thread_tests = FALSE;
 	const char        *orb_name;
 	int                i;
 
